@@ -9,7 +9,7 @@
  *
  * Written by Russ Allbery <rra@stanford.edu>
  * Based on work by Anton Ushakov
- * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010
+ * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2010, 2012
  *     The Board of Trustees of the Leland Stanford Junior University
  *
  * See LICENSE for licensing terms.
@@ -68,7 +68,7 @@ internal_v1_commandv(struct remctl *r, const struct iovec *command,
 
     /* Send the result. */
     status = token_send_priv(r->fd, r->context, TOKEN_DATA | TOKEN_SEND_MIC,
-                             &token, &major, &minor);
+                             &token, r->timeout, &major, &minor);
     if (status != TOKEN_OK) {
         internal_token_error(r, "sending token", status, major, minor);
         free(token.value);
@@ -113,10 +113,10 @@ internal_v1_output(struct remctl *r)
 
     /* Otherwise, we have to read the token from the server. */
     status = token_recv_priv(r->fd, r->context, &flags, &token,
-                             TOKEN_MAX_LENGTH, &major, &minor);
+                             TOKEN_MAX_LENGTH, r->timeout, &major, &minor);
     if (status != TOKEN_OK) {
         internal_token_error(r, "receiving token", status, major, minor);
-        if (status == TOKEN_FAIL_EOF) {
+        if (status == TOKEN_FAIL_EOF || status == TOKEN_FAIL_TIMEOUT) {
             socket_close(r->fd);
             r->fd = INVALID_SOCKET;
         }
