@@ -1,6 +1,9 @@
 /*
  * daemon test suite.
  *
+ * The canonical version of this file is maintained in the rra-c-util package,
+ * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
+ *
  * Written by Russ Allbery <rra@stanford.edu>
  *
  * The authors hereby relinquish any claim to any copyright that they may have
@@ -125,7 +128,8 @@ main(void)
     if (child < 0)
         sysbail("cannot fork");
     else if (child == 0) {
-        daemon(0, 0);
+        if (daemon(0, 0) != 0)
+            sysbail("daemon failed");
         if (chdir(start) != 0)
             sysbail("cannot chdir to %s", start);
         ok(0, "output from child that should be hidden");
