@@ -2,7 +2,7 @@
  * Test suite for address binding in the server.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2011
+ * Copyright 2011, 2012
  *     The Board of Trustees of the Leland Stanford Junior University
  *
  * See LICENSE for licensing terms.
@@ -11,17 +11,11 @@
 #include <config.h>
 #include <portable/system.h>
 
-#include <signal.h>
-#include <sys/wait.h>
-
 #include <client/remctl.h>
 #include <tests/tap/basic.h>
 #include <tests/tap/kerberos.h>
 #include <tests/tap/remctl.h>
-#include <util/concat.h>
-#include <util/messages.h>
 #include <util/network.h>
-#include <util/xmalloc.h>
 
 
 /*
@@ -56,7 +50,7 @@ test_command(struct remctl *r, const char *addr)
     const char *command[] = { "test", "env", "REMOTE_ADDR", NULL };
 
     if (!remctl_command(r, command)) {
-        notice("# remctl error %s", remctl_error(r));
+        diag("remctl error %s", remctl_error(r));
         ok_block(0, 3, "... command failed");
         return;
     }
@@ -65,9 +59,7 @@ test_command(struct remctl *r, const char *addr)
         switch (output->type) {
         case REMCTL_OUT_OUTPUT:
             is_int(strlen(addr) + 1, output->length, "... length ok");
-            seen = xmalloc(output->length);
-            memcpy(seen, output->data, output->length);
-            seen[output->length - 1] = '\0';
+            seen = bstrndup(output->data, output->length - 1);
             is_string(addr, seen, "... REMOTE_ADDR correct");
             free(seen);
             break;
@@ -75,12 +67,12 @@ test_command(struct remctl *r, const char *addr)
             is_int(0, output->status, "... status ok");
             break;
         case REMCTL_OUT_ERROR:
-            notice("# test env returned error: %.*s", (int) output->length,
-                   output->data);
+            diag("test env returned error: %.*s", (int) output->length,
+                 output->data);
             ok_block(0, 3, "... error received");
             break;
         case REMCTL_OUT_DONE:
-            notice("# unexpected done token");
+            diag("unexpected done token");
             break;
         }
     } while (output->type == REMCTL_OUT_OUTPUT);
@@ -90,37 +82,30 @@ test_command(struct remctl *r, const char *addr)
 int
 main(void)
 {
-    const char *principal;
-    char *config, *path;
+    struct kerberos_config *config;
     struct remctl *r;
-    pid_t remctld;
 #ifdef HAVE_INET6
     bool ipv6 = false;
 #endif
 
     /* Unless we have Kerberos available, we can't really do anything. */
-    if (chdir(getenv("SOURCE")) < 0)
-        bail("can't chdir to SOURCE");
-    principal = kerberos_setup();
-    if (principal == NULL)
-        skip_all("Kerberos tests not configured");
-    config = concatpath(getenv("SOURCE"), "data/conf-simple");
-    path = concatpath(getenv("BUILD"), "../server/remctld");
+    config = kerberos_setup(TAP_KRB_NEEDS_KEYTAB);
 
     /* Initialize our testing. */
     ipv6 = have_ipv6();
     plan(26);
 
     /* Test connecting to IPv4 and IPv6 with default bind. */
-    remctld = remctld_start(path, principal, config, NULL);
+    remctld_start(config, "data/conf-simple", NULL);
     r = remctl_new();
-    ok(remctl_open(r, "127.0.0.1", 14373, principal), "Connect to 127.0.0.1");
+    ok(remctl_open(r, "127.0.0.1", 14373, config->principal),
+       "Connect to 127.0.0.1");
     test_command(r, "127.0.0.1");
     remctl_close(r);
 #ifdef HAVE_INET6
     if (ipv6) {
         r = remctl_new();
-        ok(remctl_open(r, "::1", 14373, principal), "Connect to ::1");
+        ok(remctl_open(r, "::1", 14373, config->principal), "Connect to ::1");
         test_command(r, "::1");
         remctl_close(r);
     } else {
@@ -129,19 +114,19 @@ main(void)
 #else
     skip_block(4, "IPv6 not supported");
 #endif
-    remctld_stop(remctld);
+    remctld_stop();
 
     /* Try binding to only IPv4. */
-    remctld = remctld_start(path, principal, config, "-b", "127.0.0.1", NULL);
+    remctld_start(config, "data/conf-simple", "-b", "127.0.0.1", NULL);
     r = remctl_new();
-    ok(remctl_open(r, "127.0.0.1", 14373, principal),
+    ok(remctl_open(r, "127.0.0.1", 14373, config->principal),
        "Connect to 127.0.0.1 when bound to that address");
     test_command(r, "127.0.0.1");
     remctl_close(r);
 #ifdef HAVE_INET6
     if (ipv6) {
         r = remctl_new();
-        ok(!remctl_open(r, "::1", 14373, principal),
+        ok(!remctl_open(r, "::1", 14373, config->principal),
            "Cannot connect to ::1 when only bound to 127.0.0.1");
         remctl_close(r);
     } else {
@@ -150,22 +135,22 @@ main(void)
 #else
     skip("IPv6 not supported");
 #endif
-    remctld_stop(remctld);
+    remctld_stop();
 
     /* Try binding to only IPv6. */
 #ifdef HAVE_INET6
     if (ipv6) {
-        remctld = remctld_start(path, principal, config, "-b", "::1", NULL);
+        remctld_start(config, "data/conf-simple", "-b", "::1", NULL);
         r = remctl_new();
-        ok(!remctl_open(r, "127.0.0.1", 14373, principal),
+        ok(!remctl_open(r, "127.0.0.1", 14373, config->principal),
            "Cannot connect to 127.0.0.1 when only bound to ::1");
         remctl_close(r);
         r = remctl_new();
-        ok(remctl_open(r, "::1", 14373, principal),
+        ok(remctl_open(r, "::1", 14373, config->principal),
            "Connect to ::1 when bound only to it");
         test_command(r, "::1");
         remctl_close(r);
-        remctld_stop(remctld);
+        remctld_stop();
     } else {
         skip_block(5, "IPv6 not supported");
     }
@@ -176,19 +161,19 @@ main(void)
     /* Try binding explicitly to local IPv4 and IPv6 addresses. */
 #ifdef HAVE_INET6
     if (ipv6) {
-        remctld = remctld_start(path, principal, config, "-b", "127.0.0.1",
-                                "-b", "::1", NULL);
+        remctld_start(config, "data/conf-simple", "-b", "127.0.0.1", "-b",
+                      "::1", NULL);
         r = remctl_new();
-        ok(remctl_open(r, "127.0.0.1", 14373, principal),
+        ok(remctl_open(r, "127.0.0.1", 14373, config->principal),
            "Connect to 127.0.0.1 when bound to both local addresses");
         test_command(r, "127.0.0.1");
         remctl_close(r);
         r = remctl_new();
-        ok(remctl_open(r, "::1", 14373, principal),
+        ok(remctl_open(r, "::1", 14373, config->principal),
            "Connect to ::1 when bound to both local addresses");
         test_command(r, "::1");
         remctl_close(r);
-        remctld_stop(remctld);
+        remctld_stop();
     } else {
         skip_block(8, "IPv6 not supported");
     }
