@@ -3,7 +3,7 @@
  *
  * Originally written by Anton Ushakov
  * Extensive modifications by Russ Allbery <rra@stanford.edu>
- * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+ * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2012
  *     The Board of Trustees of the Leland Stanford Junior University
  *
  * See LICENSE for licensing terms.
@@ -37,7 +37,8 @@ enum token_status {
     TOKEN_FAIL_INVALID = -3,    /* Invalid token from remote site */
     TOKEN_FAIL_LARGE   = -4,    /* Token data exceeds max length */
     TOKEN_FAIL_EOF     = -5,    /* Unexpected end of file while reading */
-    TOKEN_FAIL_GSSAPI  = -6     /* GSS-API failure {en,de}crypting token */
+    TOKEN_FAIL_GSSAPI  = -6,    /* GSS-API failure {en,de}crypting token */
+    TOKEN_FAIL_TIMEOUT = -7     /* Timeout sending or receiving token */
 };
 
 BEGIN_DECLS
@@ -50,9 +51,10 @@ BEGIN_DECLS
  * token returned by token_recv; this will cause crashes on Windows.  Call
  * free on the value member instead.
  */
-enum token_status token_send(socket_type, int flags, gss_buffer_t);
+enum token_status token_send(socket_type, int flags, gss_buffer_t,
+                             time_t timeout);
 enum token_status token_recv(socket_type, int *flags, gss_buffer_t,
-                             size_t max);
+                             size_t max, time_t timeout);
 
 /* Undo default visibility change. */
 #pragma GCC visibility pop
